/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef LORENTZIAN_H
#define LORENTZIAN_H

// (Amp / PI) * (Gamma / 2) / ( (x - Mean)^2 + (Gamma / 2)^2 )

#include "QUB_BasicCurve.h"

typedef struct {
	double Amp, Mean, Gamma;
} LorentzianComponent;

static LorentzianComponent LRZ_DEFAULTS = {1.0, 1.0, 1.0};

#define LRZ_NPAR 3

static char LRZ_PARAM_NAMES[][MAX_NAMELEN] = {"Amp", "Mean", "Gamma"};

inline double squared(double x) { return x*x; }

class QUB_Curve_Lorentzian : public QUB_BasicCurve<LorentzianComponent, LRZ_NPAR>
{
public:
	QUB_Curve_Lorentzian(const char *name, IQUB_Curve *baselineCurve)
		: QUB_BasicCurve<LorentzianComponent, LRZ_NPAR>(
				name,
				"(Amp / PI) * (Gamma / 2) / ( (x - Mean)^2 + (Gamma / 2)^2 )",
				&(LRZ_PARAM_NAMES[0][0]),
				LRZ_DEFAULTS,
				baselineCurve )
	{}
	virtual void evalOne(QF_Data *data, double *yy, int c)
	{
		double x0 = components[c].Mean;
		double apg2 = (components[c].Amp / M_PI) * (components[c].Gamma / 2.0);
		double Gamma2s = squared(components[c].Gamma / 2);
		int n = data->n;
		double *xx = data->xx;
		for (int i=0; i<n; ++i)
			yy[i] += apg2 / ( squared(xx[i] - x0) + Gamma2s );
	}
};

// make your own factory function with this signature,
// returning a new instance of your class:

// vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
IQUB_Curve* __stdcall CreateLorentzian();
IQUB_Curve* __stdcall CreateLorentzianLinear();
// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^



#endif

